@@ -1,7 +1,7 @@
 /* Main command processing loop.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -33,15 +33,15 @@ is what most commands require. */
 #define NORMALIZE(x)  { x = (x)<0 ? 1 : (x); }
 
 /* The length of the static message buffer. It must be larger by a factor
-	of about three of the maximum screen width as UTF-8 encoded characters
-	might take several characters per screen position. */
+   of about three of the maximum screen width as UTF-8 encoded characters
+   might take several characters per screen position. */
 
 #define MAX_MESSAGE_SIZE (1024)
 
 /* Here, given a mask represent a user flag and an integer i, we do as follows:
-	i < 0 : toggle flag;
-	i = 0 : clear flag;
-	i > 0 : set flag;
+   i < 0 : toggle flag;
+   i = 0 : clear flag;
+   i > 0 : set flag;
 */
 
 #define SET_USER_FLAG(b,i,x) {\
@@ -55,7 +55,7 @@ is what most commands require. */
 
 
 /* Converts a non-positive result from request_number() to OK if the
-function was aborted or not-a-number error if an invalid number was read. */
+   function was aborted or not-a-number error if an invalid number was read. */
 
 #define NUMERIC_ERROR(c) ((c) == ABORT ? OK : NOT_A_NUMBER)
 
@@ -64,16 +64,16 @@ function was aborted or not-a-number error if an invalid number was read. */
 
 /* This is the dispatcher of all actions that have some effect on the text.
 
-	The arguments are an action to be executed, a possible integer parameter and
-	a possible string parameter. -1 and NULL are, respectively, reserved values
-	meaning "no argument". For most operations, the integer argument is the
-	number of repetitions. When an on/off choice is required, nonzero means on,
-	zero means off, no argument means toggle.
+   The arguments are an action to be executed, a possible integer parameter and
+   a possible string parameter. -1 and NULL are, respectively, reserved values
+   meaning "no argument". For most operations, the integer argument is the
+   number of repetitions. When an on/off choice is required, nonzero means on,
+   zero means off, no argument means toggle.
 
-	If there is a string argument (i.e. p != NULL), it is assumed that the
-	action will consume p -- it ends up being free()d or stored
-	somewhere. Though efficient, this has lead to some memory leaks (can you
-	find them?). */
+   If there is a string argument (i.e. p != NULL), it is assumed that the
+   action will consume p -- it ends up being free()d or stored
+   somewhere. Though efficient, this has lead to some memory leaks (can you
+   find them?). */
 
 
 int do_action(buffer *b, action a, int c, unsigned char *p) {
@@ -127,19 +127,16 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return stop ? STOPPED : error;
 
 	case LINEDOWN_A:
-	
 		NORMALIZE(c);
 		for(i = 0; i < c && !(error = line_down(b)) && !stop; i++);
 		return stop ? STOPPED : error;
 
 	case PREVPAGE_A:
-	
 		NORMALIZE(c);
 		for(i = 0; i < c && !(error = prev_page(b)) && !stop; i++);
 		return stop ? STOPPED : error;
 
 	case NEXTPAGE_A:
-		
 		NORMALIZE(c);
 		for(i = 0; i < c && !(error = next_page(b)) && !stop; i++);
 		return stop ? STOPPED : error;
@@ -214,44 +211,34 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		for(i = 0; i < c && !(error = search_word(b, -1)) && !stop; i++);
 		return stop ? STOPPED : error;
 
-   case DELETEPREVWORD_A:
+	case DELETENEXTWORD_A:
+	case DELETEPREVWORD_A:
 		recording = b->recording;
 		b->recording = 0;
 		NORMALIZE(c);
 		delay_update();
 		start_undo_chain(b);
 		for(i = 0; i < c && !error && !stop; i++) {
-				int right_line = b->cur_line;
-				int right_pos  = b->cur_pos;
-				if(!(error = do_action(b, PREVWORD_A, 1, NULL))) {
-					int left_line = b->cur_line;
-					int left_pos  = b->cur_pos;
-						goto_line(b, right_line);
-					goto_pos(b, right_pos);
-					while(!error && !stop && (b->cur_line > left_line || b->cur_pos > left_pos)) {
-						error = do_action(b, BACKSPACE_A, 1, NULL);
-					}
-				}
-			}
-		end_undo_chain(b);
-		b->recording = recording;
-		return stop ? STOPPED : error;
+			int marking_t = b->marking;
+			int mark_is_vertical_t = b->mark_is_vertical;
+			b->bookmark[WORDWRAP_BOOKMARK].pos   = b->block_start_pos;
+			b->bookmark[WORDWRAP_BOOKMARK].line  = b->block_start_line;
+			b->bookmark_mask                    |= (1 << WORDWRAP_BOOKMARK);
 
-   case DELETENEXTWORD_A:
-		recording = b->recording;
-		b->recording = 0;
-		NORMALIZE(c);
-		delay_update();
-		start_undo_chain(b);
-		for(i = 0; i < c && !error && !stop; i++) {
-				int left_line = b->cur_line;
-				int left_pos  = b->cur_pos;
-				if(!(error = do_action(b, NEXTWORD_A, 1, NULL))) {
-					while(!error && !stop && (b->cur_line > left_line || b->cur_pos > left_pos)) {
-						error = do_action(b, BACKSPACE_A, 1, NULL);
-					}
-				}
+			b->marking = 1;
+			b->mark_is_vertical = 0;
+			b->block_start_line = b->cur_line;
+			b->block_start_pos  = b->cur_pos;
+
+			if(!(error = do_action(b, a == DELETENEXTWORD_A ? NEXTWORD_A : PREVWORD_A, 1, NULL))) {
+				if (!(error = erase_block(b))) update_window_lines(b, b->cur_y, ne_lines - 2, FALSE);
 			}
+			b->bookmark_mask   &= ~(1 << WORDWRAP_BOOKMARK);
+			b->block_start_pos  = b->bookmark[WORDWRAP_BOOKMARK].pos;
+			b->block_start_line = b->bookmark[WORDWRAP_BOOKMARK].line;
+			b->marking          = marking_t;
+			b->mark_is_vertical = mark_is_vertical_t;
+		}
 		end_undo_chain(b);
 		b->recording = recording;
 		return stop ? STOPPED : error;
@@ -275,24 +262,25 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			free(p);
 			return OK;
 		} /* Intentionally fall through to regular BOOKMARK parm parsing. */
+
 	case SETBOOKMARK_A:
 	case GOTOBOOKMARK_A:
 		{
-			int relative = 0;
+			int relative = FALSE;
 			/* *p can be  "", "-", "0".."9", "+1","-1", for which, respectively, */
-			/*  c becomes  1,  0,   1 .. 10, next,prev. Anything else is out of range. */
+			/*  c becomes  0, AB,   0 .. 9,  next,prev. Anything else is out of range. */
 			if (p) {
 				if ((p[0]=='+' || p[0]=='-') && p[1]=='1') {
-					if (b->cur_bookmark<1 || b->cur_bookmark>=NUM_BOOKMARKS) b->cur_bookmark = 1;
-					for (i=0; i<NUM_BOOKMARKS-1; i++) {
-						b->cur_bookmark = (b->cur_bookmark-1+NUM_BOOKMARKS-1+(p[0]=='+'?1:-1))%(NUM_BOOKMARKS-1)+1;
+					if (b->cur_bookmark<0 || b->cur_bookmark>MAX_USER_BOOKMARK) b->cur_bookmark = 0;
+					for (i=0; i<=MAX_USER_BOOKMARK; i++) {
+						b->cur_bookmark = (b->cur_bookmark+MAX_USER_BOOKMARK+1+(p[0]=='+'?1:-1))%(MAX_USER_BOOKMARK+1);
 						if ((a==SETBOOKMARK_A?~b->bookmark_mask:b->bookmark_mask) & (1<<b->cur_bookmark)) {
 							c = b->cur_bookmark;
-							relative = 1;
+							relative = TRUE;
 							break;
 						}
 					}
-					if (i==NUM_BOOKMARKS-1) {
+					if (i==MAX_USER_BOOKMARK+1) {
 						free(p);
 						switch (a) {
 						case SETBOOKMARK_A:
@@ -305,17 +293,17 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 					}
 				} else if (p[0]) {
 					if (!p[1]) {
-						if (*p == '-') c = 0;
-						else c = *p - '0' + 1;
+						if (*p == '-') c = AUTO_BOOKMARK;
+						else c = *p - '0';
 					}
 					else c = -1;
 				}
-				else c = 1;
+				else c = 0;
 				free(p);
-				if (c < 0 || c >= NUM_BOOKMARKS) return INVALID_BOOKMARK_DESIGNATION;
+				if (c < 0 || c > AUTO_BOOKMARK) return INVALID_BOOKMARK_DESIGNATION;
 			}
 			else 
-				c = 1;
+				c = 0;
 			switch(a) {
 			case SETBOOKMARK_A:
 				b->bookmark[c].pos = b->cur_pos;
@@ -323,14 +311,14 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 				b->bookmark[c].cur_y = b->cur_y;
 				b->bookmark_mask |= (1 << c);
 				b->cur_bookmark = c;
-				snprintf(msg, MAX_MESSAGE_SIZE, "Bookmark %c set", c>0?'0'+c-1 : '-');
+				snprintf(msg, MAX_MESSAGE_SIZE, "Bookmark %c set", c<=MAX_USER_BOOKMARK?'0'+c : '-');
 				print_message(msg);
 				break;
 			case UNSETBOOKMARK_A:
 				if (! (b->bookmark_mask & (1 << c)))
 					return BOOKMARK_NOT_SET;
 				b->bookmark_mask &= ~(1 << c);
-				snprintf(msg, MAX_MESSAGE_SIZE, "Bookmark %c unset", c>0?'0'+c-1 : '-');
+				snprintf(msg, MAX_MESSAGE_SIZE, "Bookmark %c unset", c<=MAX_USER_BOOKMARK?'0'+c : '-');
 				print_message(msg);
 				break;
 			case GOTOBOOKMARK_A:
@@ -348,12 +336,12 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 						snprintf(msg, MAX_MESSAGE_SIZE, "%c%d", avshift > 0 ? 'T' :'B', avshift > 0 ? avshift : -avshift);
 						adjust_view(b,msg);
 					}
-					b->bookmark[0].line  = prev_line;
-					b->bookmark[0].pos   = prev_pos;
-					b->bookmark[0].cur_y = cur_y;
-					b->bookmark_mask |= 1;
+					b->bookmark[AUTO_BOOKMARK].line  = prev_line;
+					b->bookmark[AUTO_BOOKMARK].pos   = prev_pos;
+					b->bookmark[AUTO_BOOKMARK].cur_y = cur_y;
+					b->bookmark_mask |= 1<<AUTO_BOOKMARK;
 					if (relative) {
-						snprintf(msg, MAX_MESSAGE_SIZE, "At Bookmark %c", c>0?'0'+c-1 : '-');
+						snprintf(msg, MAX_MESSAGE_SIZE, "At Bookmark %c", c<=MAX_USER_BOOKMARK?'0'+c : '-');
 						print_message(msg);
 					}
 				}
@@ -367,7 +355,6 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		goto_line(b, --c);
 		return OK;
 	
-	
 	case GOTOCOLUMN_A:
 		if (c < 0 && (c = request_number("Column", b->cur_x + b->win_x + 1))<0) return NUMERIC_ERROR(c);
 		goto_column(b, c ? --c : 0);
@@ -384,7 +371,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			start_undo_chain(b);
 			
 			/* We cannot rely on encoding promotion done by INSERTCHAR_A, because it could work
-				just for part of the string if UTF-8 auto-detection is not enabled. */
+			   just for part of the string if UTF-8 auto-detection is not enabled. */
 			
 			if (b->encoding == ENC_ASCII || encoding == ENC_ASCII || (b->encoding == encoding)) {
 				if (b->encoding == ENC_ASCII) b->encoding = encoding;
@@ -395,12 +382,16 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			free(p);
 		}
 		b->recording = recording;
-		return error;	
+		return error;
 
 	case TABS_A:
 		SET_USER_FLAG(b, c, opt.tabs);
 		return OK;
 
+	case DELTABS_A:
+		SET_USER_FLAG(b, c, opt.del_tabs);
+		return OK;
+
 	case SHIFTTABS_A:
 		SET_USER_FLAG(b, c, opt.shift_tabs);
 		return OK;
@@ -432,7 +423,6 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return error;
 
 	case INSERTCHAR_A: {
-
 		static int last_inserted_char = ' ';
 		int deleted_char, old_char, error = ERROR;
 
@@ -469,7 +459,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		need_attr_update = TRUE;
 		
 		/* At this point the line has been modified: note that if we are in overwrite mode and write a character
-			at or beyond the length of the current line, we are actually doing an insertion. */
+		   at or beyond the length of the current line, we are actually doing an insertion. */
 		
 		if (!deleted_char) update_inserted_char(b, c, b->cur_line_desc, b->cur_pos, b->cur_char, b->cur_y, b->cur_x);
 		else update_overwritten_char(b, old_char, c, b->cur_line_desc, b->cur_pos, b->cur_char, b->cur_y, b->cur_x);
@@ -477,42 +467,10 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		char_right(b);
 
 		/* Note the use of ne_columns-1. This avoids a double horizontal scrolling each time a
-			word wrap happens with b->opt.right_margin = 0. */
-
-		if (b->opt.word_wrap && b->win_x + b->cur_x >= (b->opt.right_margin ? b->opt.right_margin : ne_columns - 1)) error = word_wrap(b);
-
-
-		if (error == ERROR) {
-			assert_buffer_content(b);
-			/* No word wrap. */
-			if (b->syn) update_line(b, b->cur_y, TRUE, FALSE);
-			assert_buffer_content(b);
-		}		
-		else {		
-			/* Fixes in case of word wrapping. */
-			int wont_scroll = b->win_x == 0;
-			int a = 0;
-			if (b->syn) update_line(b, b->cur_y, FALSE, TRUE);
-			else update_partial_line(b, b->cur_y, calc_width(b->cur_line_desc, b->cur_line_desc->line_len, b->opt.tab_size, b->encoding) - b->win_x, FALSE, FALSE);	
-
-			need_attr_update = FALSE;
-			/* Poke the correct state into the next line. */
-			if (b->syn) ((line_desc *)b->cur_line_desc->ld_node.next)->highlight_state = b->next_state;
-
-			if (b->opt.auto_indent) a = auto_indent_line(b, b->cur_line + 1, (line_desc *)b->cur_line_desc->ld_node.next, INT_MAX);
-			move_to_sol(b);
-			line_down(b);
-			goto_pos(b, error + a);
-                                                               
-			if (wont_scroll) {
-				if (b->cur_line == b->num_lines - 1) update_line(b, b->cur_y, FALSE, FALSE);
-				else scroll_window(b, b->cur_y, 1);
-			}
-
-			need_attr_update = TRUE;
-			assert_buffer_content(b);
-		}
+		   word wrap happens with b->opt.right_margin = 0. */
 
+		if (b->opt.word_wrap && b->win_x + b->cur_x >= (b->opt.right_margin ? b->opt.right_margin : ne_columns - 1)) word_wrap(b);
+		if (b->syn) update_line(b, b->cur_y, TRUE, FALSE);
 		assert_buffer_content(b);
 		return OK;
 	}
@@ -520,11 +478,8 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 	case BACKSPACE_A:
 	case DELETECHAR_A:
-		
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
-
 		NORMALIZE(c);
-		
 		start_undo_chain(b);
 		for(i = 0; i < c && !stop; i++) {
 			if (a == BACKSPACE_A) {
@@ -535,34 +490,35 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 						return ERROR;
 					}
 					/* We turn a backspace at the start of a line into a delete 
-						at the end of the previous line. */
+					   at the end of the previous line. */
 					char_left(b);
 				}
 				else {
-					if (!b->opt.tabs && (b->win_x + b->cur_x) % b->opt.tab_size == 0
+					if (b->opt.del_tabs && (b->win_x + b->cur_x) % b->opt.tab_size == 0
 						&& (b->cur_pos > b->cur_line_desc->line_len || b->cur_line_desc->line[b->cur_pos - 1] == ' ')) {
 						/* We are deleting one or more spaces from a tabbing position. We go left until the
-						previous tabbing, or when spaces end. */
+						   previous tabbing, or when spaces end. */
 						do char_left(b); while((b->win_x + b->cur_x) % b->opt.tab_size != 0 
-															&& (b->cur_pos > b->cur_line_desc->line_len || b->cur_line_desc->line[b->cur_pos - 1] == ' '));
+						                        && (b->cur_pos > b->cur_line_desc->line_len || b->cur_line_desc->line[b->cur_pos - 1] == ' '));
 					}
 					else char_left(b);
 					/* If we are not over text, we are in free form mode; the backspace
-						is turned into moving to the left. */
+					   is turned into moving to the left. */
 					if (b->cur_pos >= b->cur_line_desc->line_len) continue;
 				}
 			}
 			
 			/* From here, we just implement a delete. */
 			
-			if (!b->opt.tabs && b->cur_pos < b->cur_line_desc->line_len && b->cur_line_desc->line[b->cur_pos] == ' ' && 
+			if (b->opt.del_tabs && b->cur_pos < b->cur_line_desc->line_len && b->cur_line_desc->line[b->cur_pos] == ' ' && 
 				((b->win_x + b->cur_x) % b->opt.tab_size == 0 || b->cur_line_desc->line[b->cur_pos - 1] != ' ')) {
 				col = 0;
 				do col++; while((b->win_x + b->cur_x + col) % b->opt.tab_size != 0 
-					&& b->cur_pos + col < b->cur_line_desc->line_len && b->cur_line_desc->line[b->cur_pos + col] == ' ');
+				                && b->cur_pos + col < b->cur_line_desc->line_len
+				                && b->cur_line_desc->line[b->cur_pos + col] == ' ');
 				/* We are positioned at the start of the block of col spaces. If there is at most
-					one character to delete, we can just go on. Otherwise, we replace the block with a 
-					TAB, doing some magick to keep everything in sync. */
+				   one character to delete, we can just go on. Otherwise, we replace the block with a 
+				   TAB, doing some magick to keep everything in sync. */
 				if (col > 1 && (b->win_x + b->cur_x + col) % b->opt.tab_size == 0) {
 					if (b->syn) {
 						freeze_attributes(b, b->cur_line_desc);
@@ -572,7 +528,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 					}
 					delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, col);
 					insert_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos, '\t');
-					if (b->syn) update_partial_line(b, b->cur_y, b->cur_x, TRUE, TRUE);	
+					if (b->syn) update_partial_line(b, b->cur_y, b->cur_x, TRUE, TRUE);
 				}
 			}
 
@@ -600,7 +556,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 				const int old_attr = b->syn ? b->attr_buf[b->cur_pos] : 0;
 				delete_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos);
 				
-				update_deleted_char(b, old_char, old_attr, b->cur_line_desc, b->cur_pos, b->cur_char, b->cur_y, b->cur_x);	
+				update_deleted_char(b, old_char, old_attr, b->cur_line_desc, b->cur_pos, b->cur_char, b->cur_y, b->cur_x);
 				if (b->syn) update_line(b, b->cur_y, TRUE, TRUE);
 			}
 			else {
@@ -614,11 +570,11 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 				if (b->syn) {
 					b->next_state = parse(b->syn, b->cur_line_desc, b->cur_line_desc->highlight_state, b->encoding == ENC_UTF8); 
 					update_line(b, b->cur_y, FALSE, TRUE);
-				}	
+				}
 				else update_partial_line(b, b->cur_y, b->cur_x, TRUE, FALSE);
-					
+				
 				if (b->cur_y < ne_lines - 2) scroll_window(b, b->cur_y + 1, -1);
-			}			
+			}
 		}
 		need_attr_update = TRUE;
 		end_undo_chain(b);
@@ -648,7 +604,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 					move_to_sol(b);
 					line_down(b);
-					if (a != -1) goto_pos(b, a);					
+					if (a != -1) goto_pos(b, a);
 				}
 				else {
 					int a = -1;
@@ -664,7 +620,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 					move_to_sol(b);
 					line_down(b);
-					if (a != -1) goto_pos(b, a);					
+					if (a != -1) goto_pos(b, a);
 
 					if (b->cur_line == b->num_lines - 1) update_line(b, b->cur_y, FALSE, FALSE);
 					else scroll_window(b, b->cur_y, 1);
@@ -708,15 +664,15 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		start_undo_chain(b);
 		for(i = 0; i < c && !stop; i++) {
 			/* This is a bit tricky. First of all, if we are undeleting for
-				the first time and the local attribute buffer is not valid
-				we fill it. */
+			   the first time and the local attribute buffer is not valid
+			   we fill it. */
 			if (i == 0 && b->syn && b->attr_len < 0) freeze_attributes(b, b->cur_line_desc);
 			if (error = undelete_line(b)) break;
 			if (i == 0) {
 				if (b->syn) {
 					/* Now the only valid part of the local attribute buffer is before b->cur_pos. 
-						We perform a differential update so that if we undelete in the middle of
-						a line we avoid to rewrite the part up to b->cur_pos. */
+					   We perform a differential update so that if we undelete in the middle of
+					   a line we avoid to rewrite the part up to b->cur_pos. */
 					b->attr_len = b->cur_pos;
 					update_line(b, b->cur_y, FALSE, TRUE);
 					next_line_state = b->next_state;
@@ -814,10 +770,10 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 				if (b->opt.auto_prefs && extension(p)) load_auto_prefs(b, extension(p));
 				error = load_file_in_buffer(b, p);
 				if (error != FILE_IS_MIGRATED 
-					&& error != FILE_IS_DIRECTORY 
-					&& error != IO_ERROR 
-					&& error != FILE_IS_TOO_LARGE 
-					&& error != OUT_OF_MEMORY) change_filename(b, p);
+				   && error != FILE_IS_DIRECTORY 
+				   && error != IO_ERROR 
+				   && error != FILE_IS_TOO_LARGE 
+				   && error != OUT_OF_MEMORY) change_filename(b, p);
 				print_error(error);
 				reset_window();
 				return OK;
@@ -828,9 +784,9 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return ERROR;
 
 	case ABOUT_A:
-	   about(1);
-	   c = get_key_code();
-	   about(0);
+		about(1);
+		c = get_key_code();
+		about(0);
 		return OK;
 
 	case REFRESH_A:
@@ -905,7 +861,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 				if (a == REPLACEALL_A) start_undo_chain(b);
 
 				while(!stop && 
-						!(error = (b->last_was_regexp ? find_regexp : find)(b, NULL, !first_search && a != REPLACEALL_A && c != 'A' && c != 'Y'))) {
+				      !(error = (b->last_was_regexp ? find_regexp : find)(b, NULL, !first_search && a != REPLACEALL_A && c != 'A' && c != 'Y'))) {
 
 					if (c != 'A' && a != REPLACEALL_A && a != REPLACEONCE_A) {
 						refresh_window(b);
@@ -933,7 +889,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 							if (last_replace_empty_match)
 								if (b->opt.search_back) error = char_left(cur_buffer);
 								else error = char_right(cur_buffer);
-						}					
+						}
 	
 						if (print_error(error)) {
 							if (a == REPLACEALL_A || c == 'A') end_undo_chain(b);
@@ -959,7 +915,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 				}
 				if (stop) return STOPPED;
 
-				if ((c != 'A' && a != REPLACEALL_A || first_search) && error || error != NOT_FOUND) {
+				if (error && ((c != 'A' && a != REPLACEALL_A || first_search) || error != NOT_FOUND )) {
 					print_error(error);
 					return ERROR;
 				}
@@ -980,7 +936,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 			if (b->last_was_replace) {
 				const encoding_type replace_encoding = detect_encoding(b->replace_string, strlen(b->replace_string));
 				if (replace_encoding != ENC_ASCII && b->encoding != ENC_ASCII && replace_encoding != b->encoding ||
-					search_encoding != ENC_ASCII && replace_encoding != ENC_ASCII && search_encoding != replace_encoding)
+				    search_encoding != ENC_ASCII && replace_encoding != ENC_ASCII && search_encoding != replace_encoding)
 					return INCOMPATIBLE_REPLACE_STRING_ENCODING;
 			}
 
@@ -1002,7 +958,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 							if (last_replace_empty_match)
 								if (b->opt.search_back) error = char_left(cur_buffer);
 								else error = char_right(cur_buffer);
-						}	
+						}
 
 						if (print_error(error)) {
 							return_code = ERROR;
@@ -1276,7 +1232,7 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		reset_window();
 
 		/* We always return ERROR after a buffer has been deleted. Otherwise,
-			the calling routines (and macros) could work on an unexisting buffer. */
+		   the calling routines (and macros) could work on an unexisting buffer. */
 
 		return ERROR;
 
@@ -1458,11 +1414,8 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return ERROR;
 
 	case TOUPPER_A:
-
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
-
 		NORMALIZE(c);
-
 		start_undo_chain(b);
 		for(i = 0; i < c && !(error = to_upper(b)) && !stop; i++);
 		end_undo_chain(b);
@@ -1470,11 +1423,8 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return print_error(error) ? ERROR : 0;
 
 	case TOLOWER_A:
-
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
-
 		NORMALIZE(c);
-
 		start_undo_chain(b);
 		for(i = 0; i < c && !(error = to_lower(b)) && !stop; i++);
 		end_undo_chain(b);
@@ -1482,11 +1432,8 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 		return print_error(error) ? ERROR : 0;
 
 	case CAPITALIZE_A:
-
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
-
 		NORMALIZE(c);
-
 		start_undo_chain(b);
 		for(i = 0; i < c && !(error = capitalize(b)) && !stop; i++);
 		end_undo_chain(b);
@@ -1495,11 +1442,8 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 
 	case CENTER_A:
-
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
-
 		NORMALIZE(c);
-
 		start_undo_chain(b);
 		for(i = 0; i < c && !(error = center(b)) && !stop; i++) {
 			need_attr_update = TRUE;
@@ -1515,17 +1459,13 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 	case PARAGRAPH_A:
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
-
 		NORMALIZE(c);
-
 		for(i = 0; i < c && !(error = paragraph(b)) && !stop; i++);
-
 		if (stop) error = STOPPED;
 		return print_error(error) ? ERROR : 0;
 
 	case SHIFT_A:
 		if (b->opt.read_only) return FILE_IS_READ_ONLY;
-
 		error = shift(b, p, &msg[0], MAX_MESSAGE_SIZE);
 		if (stop) error = STOPPED;
 		if (p) free(p);
@@ -1621,18 +1561,18 @@ int do_action(buffer *b, action a, int c, unsigned char *p) {
 
 	case AUTOCOMPLETE_A:
 		/* Since we are going to call other actions (INSERTSTRING_A and DELETEPREVWORD_A),
-			we do not want to record this insertion twice. Also, we are counting on 
-			INSERTSTRING_A to handle character encoding issues. */
+		   we do not want to record this insertion twice. Also, we are counting on 
+		   INSERTSTRING_A to handle character encoding issues. */
 		recording = b->recording;
 
 		i = b->cur_pos;
 		
 		if ( !p ) { /* no prefix given; find one left of the cursor. */
 			if ( context_prefix(b, &p, &i, b->encoding) ) return OUT_OF_MEMORY;
-		}	
+		}
       
 		snprintf(msg, MAX_MESSAGE_SIZE, "AutoComplete: prefix \"%s\"", p);
-			
+		
 		if (p = autocomplete(p, msg, TRUE, &col)) {
 			b->recording = 0;
 			start_undo_chain(b);
