@@ -1,34 +1,34 @@
 /* Clip handling functions.
 
-	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
+   Copyright (C) 1993-1998 Sebastiano Vigna 
+   Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
 
-	This file is part of ne, the nice editor.
+   This file is part of ne, the nice editor.
 
-	This library is free software; you can redistribute it and/or modify it
-	under the terms of the GNU General Public License as published by
-	the Free Software Foundation; either version 3 of the License, or (at your
-	option) any later version.
+   This library is free software; you can redistribute it and/or modify it
+   under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 3 of the License, or (at your
+   option) any later version.
 
-	This library is distributed in the hope that it will be useful, but
-	WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
-	or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
-	for more details.
+   This library is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+   or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+   for more details.
 
-	You should have received a copy of the GNU General Public License
-	along with this program; if not, see <http://www.gnu.org/licenses/>.  */
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, see <http://www.gnu.org/licenses/>.  */
 
 
 #include "ne.h"
 
 
 /* A clip is a numbered node in the global clip list. The contents of the clip
-	are handled through the stream functions contained in streams.c.
+   are handled through the stream functions contained in streams.c.
 
-	At creation time, a clip is marked with an encoding. Clips, of course, may
-	be pasted only in buffers with a compatible encoding. 
+   At creation time, a clip is marked with an encoding. Clips, of course, may
+   be pasted only in buffers with a compatible encoding. 
 
-	Note that pasting a clip in an ASCII buffer may change its encoding.
+   Note that pasting a clip in an ASCII buffer may change its encoding.
 */
 
 
@@ -53,7 +53,7 @@ clip_desc *alloc_clip_desc(int n, int size) {
 
 
 /* Reallocates a clip descriptor of the given size. If cd is NULL, this is
-	equivalent to calling alloc_clip_desc. */
+   equivalent to calling alloc_clip_desc. */
 
 clip_desc *realloc_clip_desc(clip_desc *cd, int n, int size) {
 
@@ -91,7 +91,7 @@ void free_clip_desc(clip_desc *cd) {
 
 
 /* Scans the global clip list, searching for a specific numbered clip. Returns
-	NULL on failure. */
+   NULL on failure. */
 
 clip_desc *get_nth_clip(int n) {
 	clip_desc *cd = (clip_desc *)clips.head;
@@ -110,14 +110,14 @@ clip_desc *get_nth_clip(int n) {
 
 
 /* Copies the characters between the cursor and the block marker of the given
-	buffer to the nth clip. If the cut flag is true, the characters are also
-	removed from the text. The code scans the text two times: the first time in
-	order to determine the exact length of the text, the second time in order to
-	actually copy it. */
+   buffer to the nth clip. If the cut flag is true, the characters are also
+   removed from the text. The code scans the text two times: the first time in
+   order to determine the exact length of the text, the second time in order to
+   actually copy it. */
 
 int copy_to_clip(buffer *b, int n, int cut) {
 
-	int i, pass, start_pos, end_pos, len, clip_len, y = b->cur_line;
+	int i, chaining, bsp, pass, start_pos, end_pos, len, clip_len, y = b->cur_line;
 	unsigned char *p = NULL;
 	clip_desc *cd, *new_cd;
 	line_desc *ld = b->cur_line_desc;
@@ -142,27 +142,33 @@ int copy_to_clip(buffer *b, int n, int cut) {
 
 
 	/* We have two different loops for direct or inverse copying. Making this
-	conditional code would be cumbersome, awkward, and definitely inefficient. */
+	   conditional code would be cumbersome, awkward, and definitely inefficient. */
 
 	if (y > b->block_start_line || y == b->block_start_line && b->cur_pos > b->block_start_pos) {
-
-		for(pass = clip_len = 0; pass < 2; pass++) {
+		/* mark before/above cursor */
+		for(chaining = pass = clip_len = 0; pass < 2; pass++) {
 
 			ld = b->cur_line_desc;
 
 			for(i = y; i >= b->block_start_line; i--) {
-
 				start_pos = 0;
-				len = ld->line_len;
 
 				if (i == b->block_start_line) {
-					start_pos = b->block_start_pos;
-					len -= start_pos;
+					if (!pass && cut && ld->line_len < b->block_start_pos) {
+						if (!chaining) {
+							chaining = 1;
+							start_undo_chain(b);
+						}
+						bsp = b->block_start_pos; /* because the mark will move when we insert_spaces()! */
+						insert_spaces(b, ld, i, ld->line_len, b->block_start_pos - ld->line_len);
+						b->block_start_pos = bsp;
+					}
+					start_pos = min(ld->line_len,b->block_start_pos);
 				}
 
-				if (i == y) {
-					len -= (b->cur_pos >= ld->line_len) ? 0 : ld->line_len - b->cur_pos;
-				}
+				if (i == y) end_pos = min(ld->line_len,b->cur_pos);
+				else end_pos = ld->line_len;
+				len = end_pos - start_pos;
 
 				if (pass) {
 					assert(!(len != 0 && ld->line == NULL));
@@ -177,7 +183,6 @@ int copy_to_clip(buffer *b, int n, int cut) {
 			}
 
 			if (pass) {
-
 				cd->cs->len = clip_len;
 				set_stream_encoding(cd->cs, b->encoding);
 				assert_clip_desc(cd);
@@ -189,35 +194,43 @@ int copy_to_clip(buffer *b, int n, int cut) {
 					update_syntax_and_lines(b, b->cur_line_desc, NULL);
 				}
 
+				if (chaining) end_undo_chain(b);
 				return OK;
 			}
 
-			if (!(new_cd = realloc_clip_desc(cd, n, clip_len))) return OUT_OF_MEMORY;
+			if (!(new_cd = realloc_clip_desc(cd, n, clip_len))) {
+				if (chaining) end_undo_chain(b);
+				return OUT_OF_MEMORY;
+			}
 			if (!cd) add_head(&clips, &new_cd->cd_node);
 			cd = new_cd;
 			p = cd->cs->stream + clip_len;
 		}
 	}
 	else {
-
-		for(pass = clip_len = 0; pass < 2; pass++) {
+		/* mark after cursor */
+		for(chaining = pass = clip_len = 0; pass < 2; pass++) {
 
 			ld = b->cur_line_desc;
 
 			for(i = y; i <= b->block_start_line; i++) {
 
 				start_pos = 0;
-				len = ld->line_len;
 
 				if (i == y) {
+					if (!pass && cut && b->cur_pos > ld->line_len) {
+						if (!chaining) {
+							chaining = 1;
+							start_undo_chain(b);
+						}
+						insert_spaces(b, ld, i, ld->line_len, b->cur_pos - ld->line_len);
+					}
 					start_pos = b->cur_pos > ld->line_len ? ld->line_len : b->cur_pos;
-					len -= start_pos;
 				}
 
-				if (i == b->block_start_line) {
-					end_pos = b->block_start_pos;
-					len -= ld->line_len - end_pos;
-				}
+				if (i == b->block_start_line) end_pos = min(b->block_start_pos, ld->line_len);
+				else end_pos = ld->line_len;
+				len = end_pos - start_pos;
 
 				if (pass) {
 					assert(!(len != 0 && ld->line == NULL));
@@ -237,15 +250,20 @@ int copy_to_clip(buffer *b, int n, int cut) {
 				assert_clip_desc(cd);
 				if (cut) delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, clip_len);
 				update_syntax_and_lines(b, b->cur_line_desc, NULL);
+				if (chaining) end_undo_chain(b);
 				return OK;
 			}
 
-			if (!(new_cd = realloc_clip_desc(cd, n, clip_len))) return OUT_OF_MEMORY;
+			if (!(new_cd = realloc_clip_desc(cd, n, clip_len))) {
+				if (chaining) end_undo_chain(b);
+				return OUT_OF_MEMORY;
+			}
 			if (!cd) add_head(&clips, &new_cd->cd_node);
 			cd = new_cd;
 			p = cd->cs->stream;
 		}
 	}
+	if (chaining) end_undo_chain(b);
 	return OK;
 }
 
@@ -255,52 +273,66 @@ int copy_to_clip(buffer *b, int n, int cut) {
 
 int erase_block(buffer *b) {
 
-	int i, start_pos, end_pos, len, erase_len = 0, y = b->cur_line;
+	int i, bsp, start_pos, end_pos, len, erase_len = 0, chaining = 0, y = b->cur_line;
 	line_desc *ld = b->cur_line_desc;
 
 	if (!b->marking) return MARK_BLOCK_FIRST;
 	if (b->block_start_line >= b->num_lines) return MARK_OUT_OF_BUFFER;
 
 	if (y == b->block_start_line &&
-		(b->cur_pos == b->block_start_pos ||
-		 b->cur_pos >= ld->line_len && b->block_start_pos >= ld->line_len))
+	   (b->cur_pos == b->block_start_pos ||
+	    b->cur_pos >= ld->line_len && b->block_start_pos >= ld->line_len))
 		return OK;
 
 	if (y > b->block_start_line || y == b->block_start_line && b->cur_pos > b->block_start_pos) {
 		for(i = y; i >= b->block_start_line; i--) {
 			start_pos = 0;
-			len = ld->line_len;
 			if (i == b->block_start_line) {
-				start_pos = b->block_start_pos;
-				len -= start_pos;
-			}
-			if (i == y) {
-				len -= (b->cur_pos >= ld->line_len) ? 0 : ld->line_len - b->cur_pos;
+				if (ld->line_len < b->block_start_pos) {
+					if (!chaining) {
+						chaining = 1;
+						start_undo_chain(b);
+					}
+					bsp = b->block_start_pos; /* because the mark will move when we insert_spaces()! */
+					insert_spaces(b, ld, i, ld->line_len, b->block_start_pos - ld->line_len);
+					b->block_start_pos = bsp;
+				}
+				start_pos = min(ld->line_len,b->block_start_pos);
 			}
+			if (i == y) end_pos = min(ld->line_len,b->cur_pos);
+			else end_pos = ld->line_len;
+			len = end_pos - start_pos;
+
 			erase_len += len + 1;
 			ld = (line_desc *)ld->ld_node.prev;
 		}
 		goto_line(b, b->block_start_line);
-      goto_column(b, calc_width(b->cur_line_desc, b->block_start_pos, b->opt.tab_size, b->encoding));
+		goto_column(b, calc_width(b->cur_line_desc, b->block_start_pos, b->opt.tab_size, b->encoding));
 	}
 	else {
 		for(i = y; i <= b->block_start_line; i++) {
 			start_pos = 0;
-			len = ld->line_len;
 			if (i == y) {
-				start_pos = b->cur_pos >= ld->line_len ? ld->line_len : b->cur_pos;
-				len -= start_pos;
-			}
-			if (i == b->block_start_line) {
-				end_pos = b->block_start_pos;
-				len -= ld->line_len - end_pos;
+				if (b->cur_pos > ld->line_len) {
+					if (!chaining) {
+						chaining = 1;
+						start_undo_chain(b);
+					}
+					insert_spaces(b, ld, i, ld->line_len, b->cur_pos - ld->line_len);
+				}
+				start_pos = b->cur_pos > ld->line_len ? ld->line_len : b->cur_pos;
 			}
+			if (i == b->block_start_line) end_pos = min(b->block_start_pos, ld->line_len);
+			else end_pos = ld->line_len;
+			len = end_pos - start_pos;
+
 			erase_len += len + 1;
 			ld = (line_desc *)ld->ld_node.next;
 		}
 	}
 
 	delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, erase_len - 1);
+	if (chaining) end_undo_chain(b);
 	update_syntax_and_lines(b, b->cur_line_desc, NULL);
 	return OK;
 }
@@ -309,7 +341,7 @@ int erase_block(buffer *b) {
 
 
 /* Pastes a clip into a buffer. Since clips are streams, the operation is
-	definitely straightforward. */
+   definitely straightforward. */
 
 int paste_to_buffer(buffer *b, int n) {
 
@@ -340,9 +372,9 @@ int paste_to_buffer(buffer *b, int n) {
 
 
 /* Works like copy_to_clip(), but the region to copy is the rectangle defined
-	by the cursor and the marker. Same comments apply. Note that in case of a
-	cut we use start_undo_chain() in order to make the various deletions a
-	single undo operation. */
+   by the cursor and the marker. Same comments apply. Note that in case of a
+   cut we use start_undo_chain() in order to make the various deletions a
+   single undo operation. */
 
 int copy_vert_to_clip(buffer *b, int n, int cut) {
 
@@ -357,8 +389,8 @@ int copy_vert_to_clip(buffer *b, int n, int cut) {
 	cd = get_nth_clip(n);
 
 	if (b->cur_pos == b->block_start_pos ||
-		y == b->block_start_line && b->cur_pos >= ld->line_len && 
-		b->block_start_pos >= ld->line_len) {
+	    y == b->block_start_line && b->cur_pos >= ld->line_len && 
+	    b->block_start_pos >= ld->line_len) {
 
 		if (!(new_cd = realloc_clip_desc(cd, n, 0))) return OUT_OF_MEMORY;
 		set_stream_encoding(new_cd->cs, ENC_ASCII);
@@ -385,8 +417,8 @@ int copy_vert_to_clip(buffer *b, int n, int cut) {
 
 			for(i = y; i >= b->block_start_line; i--) {
 
-			   start_pos = calc_pos(ld, start_x, b->opt.tab_size, b->encoding);
-			   len = calc_pos(ld, end_x, b->opt.tab_size, b->encoding) - start_pos;
+				start_pos = calc_pos(ld, start_x, b->opt.tab_size, b->encoding);
+				len = calc_pos(ld, end_x, b->opt.tab_size, b->encoding) - start_pos;
 
 				if (pass) {
 					*--p = 0;
@@ -475,8 +507,8 @@ int erase_vert_block(buffer *b) {
 	if (b->block_start_line >= b->num_lines) return MARK_OUT_OF_BUFFER;
 
 	if (b->cur_pos == b->block_start_pos ||
-		y == b->block_start_line && b->cur_pos >= ld->line_len && 
-		b->block_start_pos >= ld->line_len) 
+	   y == b->block_start_line && b->cur_pos >= ld->line_len && 
+	   b->block_start_pos >= ld->line_len) 
 		return OK;
 
 	start_x = calc_width(b->cur_line_desc, b->block_start_pos, b->opt.tab_size, b->encoding);
@@ -520,8 +552,8 @@ int erase_vert_block(buffer *b) {
 
 
 /* Performs a vertical paste. It has to be done via an insert_stream() for each
-	string of the clip. Again, the undo linking feature makes all these
-	operations a single undo step. */
+   string of the clip. Again, the undo linking feature makes all these
+   operations a single undo step. */
 
 int paste_vert_to_buffer(buffer *b, int n) {
 
@@ -548,7 +580,7 @@ int paste_vert_to_buffer(buffer *b, int n) {
 	while(p - stream < stream_len) {
 		if (!ld->ld_node.next) {
 			insert_one_line(b, (line_desc *)ld->ld_node.prev, line - 1, ((line_desc *)ld->ld_node.prev)->line_len);
-				ld = (line_desc *)ld->ld_node.prev;
+			ld = (line_desc *)ld->ld_node.prev;
 		}
 
 		if (len = strnlen_ne(p, stream_len - (p - stream))) {
@@ -564,20 +596,20 @@ int paste_vert_to_buffer(buffer *b, int n) {
 			}
 			else insert_stream(b, ld, line, i, p, len);
 		}
-			
+
 		p += len + 1;
 		ld = (line_desc *)ld->ld_node.next;
 		line++;
 	}
-		
+
 	end_undo_chain(b);
-	update_syntax_and_lines(b, b->cur_line_desc, ld);		
+	update_syntax_and_lines(b, b->cur_line_desc, ld);
 	return OK;
 }
 
 
 /* Loads a clip. It is just a load_stream, plus an insertion in the clip
-	list. If preserve_cr is TRUE, CRs are preserved. */
+   list. If preserve_cr is TRUE, CRs are preserved. */
 
 int load_clip(int n, const char *name, const int preserve_cr, const int binary) {
 	int error;
@@ -598,7 +630,7 @@ int load_clip(int n, const char *name, const int preserve_cr, const int binary)
 
 
 /* Saves a clip to a file. If CRLF is true, the clip is saved with CR/LF pairs
- as line terminators. */
+   as line terminators. */
 
 int save_clip(int n, const char *name, const int CRLF, const int binary) {
 
