@@ -1,7 +1,7 @@
 /* Various editing functions such as word wrap, to upper, etc.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna
-	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -85,7 +85,7 @@ static int to_something(buffer *b, int (to_first)(int), int (to_rest)(int)) {
 		start_undo_chain(b);
 
 		delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, len);
-		insert_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, word, new_len);
+		if (new_len) insert_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, word, new_len);
 
 		free(word);
 
@@ -204,46 +204,56 @@ int find_matching_bracket(buffer *b, const int min_line, int max_line, int *matc
 
 
 
-/* Breaks a line at the first possible position before the current cursor
-	position (i.e., at a tab or at a space). The space is deleted, and a new
-	line is inserted. The cursor is repositioned coherently.  The number of
-	characters existing on the new line is returned, or ERROR if no word wrap was
-	possible. */
+/* This experimental alternative to word wrapping sets a bookmark, calls
+   paragraph(), then returns to the bookmark (which may have moved due to
+   insertions/deletions).  The number of characters existing on the new line is
+   returned, or ERROR if no word wrap was possible. */
 
 int word_wrap(buffer * const b) {
-	const int len = b->cur_line_desc->line_len;
-	unsigned char * const line = b->cur_line_desc->line;
-	int cur_pos, pos, first_pos;
+	static char avcmd[16];
+	int non_blank_added, pos;
+	unsigned char * line = b->cur_line_desc->line;
+	int avshift;
 
-	if (!(cur_pos = pos = b->cur_pos)) return ERROR;
-
-	/* Find the first possible position we could break a line on. */
-	first_pos = 0;
-
-	/* Skip leading white space */
-	while (first_pos < len && ne_isspace(get_char(&line[first_pos], b->encoding), b->encoding)) first_pos = next_pos(line, first_pos, b->encoding);
-
-	/* Skip non_space after leading white space */
-	while (first_pos < len && !ne_isspace(get_char(&line[first_pos], b->encoding), b->encoding)) first_pos = next_pos(line, first_pos, b->encoding);
-
-	/* Now we know that the line shouldn't be broken before &line[first_pos]. */
-
-	/* Start from the other end now and find a candidate space to break the line on.*/
-	while((pos = prev_pos(line, pos, b->encoding)) && !ne_isspace(get_char(&line[pos], b->encoding), b->encoding));
-
-	if (! pos || pos < first_pos) return ERROR;
-
-	start_undo_chain(b);
-
-	delete_one_char(b, b->cur_line_desc, b->cur_line, pos);
-	insert_one_line(b, b->cur_line_desc, b->cur_line, pos);
-
-	end_undo_chain(b);
-
-	return b->cur_pos - pos - 1;
+	if (!b->cur_pos || b->cur_pos > b->cur_line_desc->line_len) return ERROR;
+	/* If the char to our left is a space, we need to insert
+	   something else to attach our WORDWRAP_BOOKMARK to because
+	   spaces at the split point get removed, which effectively
+	   leaves our bookmark on the current line. */
+	delay_update();
+	pos = prev_pos(line, b->cur_pos, b->encoding);
+	if (non_blank_added = ne_isspace(get_char(&line[pos], b->encoding), b->encoding)) {
+		start_undo_chain(b);
+		insert_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos, 'X');
+		line = b->cur_line_desc->line;
+		goto_pos(b, next_pos(line, b->cur_pos, b->encoding));
+		/* Get rid of any spaces starting at cur_pos */
+		pos = b->cur_pos;
+		while (pos < b->cur_line_desc->line_len && ne_isspace(get_char(&line[pos       ], b->encoding), b->encoding))
+			pos = next_pos(line, pos, b->encoding);
+		if (pos > b->cur_pos) delete_stream(b, b->cur_line_desc, b->cur_line, b->cur_pos, pos - b->cur_pos);
+	}
+	b->bookmark[WORDWRAP_BOOKMARK].pos   = b->cur_pos;
+	b->bookmark[WORDWRAP_BOOKMARK].line  = b->cur_line;
+	b->bookmark[WORDWRAP_BOOKMARK].cur_y = b->cur_y;
+	b->bookmark_mask |= (1 << WORDWRAP_BOOKMARK);
+	paragraph(b);
+	goto_line(b, b->bookmark[WORDWRAP_BOOKMARK].line);
+	goto_pos( b, b->bookmark[WORDWRAP_BOOKMARK].pos);
+	line = b->cur_line_desc->line;
+	if (avshift = b->cur_y - b->bookmark[WORDWRAP_BOOKMARK].cur_y-1) {
+		snprintf(avcmd, 16, "%c%d", avshift > 0 ? 'T' :'B', avshift > 0 ? avshift : -avshift);
+		adjust_view(b,avcmd);
+	}
+	b->bookmark_mask &= ~(1 << WORDWRAP_BOOKMARK);
+	if (non_blank_added) {
+		goto_pos(b, prev_pos(b->cur_line_desc->line, b->cur_pos, b->encoding));
+		delete_one_char(b, b->cur_line_desc, b->cur_line, b->cur_pos);
+		end_undo_chain(b);
+	}
+	return b->cur_line_desc->line_len;
 }
 
-
 /* These functions reformat a paragraph while preserving appropriate
 leading US-ASCII white space. The strategy is as follows:
 
@@ -251,6 +261,9 @@ leading US-ASCII white space. The strategy is as follows:
      following the current line if it is non-blank. Otherwise it will be
      taken from the current line. Save a copy of it for later as space[].
 
+  1.1 If the leading non-blank (the part after space[]) is not an alphanumeric,
+     then we take it as a comment initiator and preserve it for later as spots[].
+
   2. Start an undo chain.
 
   3. Trim trailing space off the current line.
@@ -261,11 +274,15 @@ leading US-ASCII white space. The strategy is as follows:
        4.3  Split the line at the split point.  (We are done with this line)
        4.4  Make the new line the current line
        4.5  Insert the space[] stream we saved in step 1.
+       4.5.1 Insert the spots[] stream we saved from step 1.1.
        4.6  Trim trailing space off the end of the line.
 
   5. If the _following_ line is part of this paragraph (i.e., its first
      non-blank character is in the correct position):
        5.1  Add a space to the end of the current line.
+       5.2  Delete this line's leading white space.
+       5.3  If the leading non-blank character matches the first character
+            of spots[], remove it any any subsequent non-alphanumeric.
        5.2  Copy following line's data starting with the first
             non-blank to the end of the current line.
        5.3  Remove the following line.
@@ -283,9 +300,32 @@ leading US-ASCII white space. The strategy is as follows:
 
 */
 
-static char *pa_space	  = NULL;  /* Where we keep space for paragraph left offsets */
-static int	pa_space_len;			/* How long pa_space is when tabs are expanded */
-static int	pa_space_pos;			/* How long pa_space is without expanding tabs */
+static char *pa_spots     = NULL;  /* Where we keep leading non-alphanumerics */
+static int   pa_spots_pos;         /* How long pa_spots is in chars */
+
+/* save_spots() is like save_space(), but it preserves the string of non-alphanumerics
+   immediately follow where save_space() left off. */
+
+static int save_spots(line_desc * const ld, const int pos, const encoding_type encoding) {
+	int rc = 0;
+	if (pa_spots) free(pa_spots);
+	pa_spots  = NULL;
+	pa_spots_pos = 0;
+	
+	if (!ld->line || ld->line_len <= pos) return 0; /* No data on this line. */
+
+	while(pos + pa_spots_pos < ld->line_len && isparaspot(ld->line[pos+pa_spots_pos]))
+		pa_spots_pos = next_pos(ld->line, pos+pa_spots_pos, encoding) - pos;
+	if (pa_spots_pos) {
+		if ((pa_spots = malloc(pa_spots_pos)))	memcpy(pa_spots, ld->line+pos, pa_spots_pos);
+		else pa_spots_pos = 0;
+	}
+	return pa_spots_pos > 0;
+}
+	
+static char *pa_space     = NULL;  /* Where we keep space for paragraph left offsets */
+static int   pa_space_len;         /* How long pa_space is when tabs are expanded */
+static int   pa_space_pos;         /* How long pa_space is without expanding tabs */
 
 /* save_space() sets pa_space, pa_space_len, and pa_space_pos to reflect the
 	space on the left end of the line ld refers to in the context of the given
@@ -297,7 +337,7 @@ static int	pa_space_pos;			/* How long pa_space is without expanding tabs */
 static int save_space(line_desc * const ld, const int tab_size, const encoding_type encoding) {
 	int pos;
 	if (pa_space) free(pa_space);
-	pa_space	  = NULL;
+	pa_space  = NULL;
 	pa_space_len = 0;
 	pa_space_pos = 0;
 
@@ -313,38 +353,38 @@ static int save_space(line_desc * const ld, const int tab_size, const encoding_t
 
 	if (pos == 0) {
 		pa_space = NULL;
+		save_spots(ld, pos, encoding);
 		return 1;
 	}
 
 	if ((pa_space = malloc(pos))) {
 		memcpy(pa_space, ld->line, pos);
+		save_spots(ld, pos, encoding);
 		return 1;
 	}
+
 	return 0;
 }
 
 
 /* trim_trailing_space() removes spaces from the end of the line referred to by
-	the line_desc ld. The int line is necessary if you want to be able to undo
-	later. */
+   the line_desc ld. The int line is necessary if you want to be able to undo
+   later. */
 
 static void trim_trailing_space(buffer * const b, line_desc *ld, const int line, const encoding_type encoding) {
 	int pos;
 	if (!ld->line) return;
-	insert_one_char(b, ld, line, ld->line_len, ' '); /* Make sure there's a space on the end and not a UTF8
-                                                       so the prev_pos() below won't go berzerk. */
 	pos = ld->line_len;
-	while(pos > 0 && isasciispace(ld->line[pos - 1])) pos = prev_pos(ld->line, pos, encoding);
-
+	while (pos > 0 && isasciispace(ld->line[pos - 1])) pos = prev_pos(ld->line, pos, encoding);
 	if (pos >= 0 && pos < ld->line_len) delete_stream(b, ld, line, pos, ld->line_len - pos);
 }
 
 /* is_part_of_paragraph() determines if the line ld refers to could be
-	considered part of a paragraph based on its leading spaces compared to
-	pa_space_len. If they are the same, is_part_of_paragraph() returns 1, and
-	*first_non_blank is set to the position of the first non-blank character on
-	the line. Otherwise, *first_non_blank is -1 and is_part_of_paragraph()
-	returns 0. */
+   considered part of a paragraph based on its leading spaces compared to
+   pa_space_len. If they are the same, is_part_of_paragraph() returns 1, and
+   *first_non_blank is set to the position of the first non-blank character on
+   the line. Otherwise, *first_non_blank is -1 and is_part_of_paragraph()
+   returns 0. */
 
 static int is_part_of_paragraph(const line_desc * const ld, const int tab_size, int * const first_non_blank, const encoding_type encoding) {
 	int pos = 0;
@@ -359,8 +399,8 @@ static int is_part_of_paragraph(const line_desc * const ld, const int tab_size,
 }
 
 /* paragraph() reformats a paragraph following the current parameters for
-	right_margin (a value of 0 forces the use of the full screen width).  On
-	completion the cursor is positioned either:
+   right_margin (a value of 0 forces the use of the full screen width).  On
+   completion the cursor is positioned either:
 
   * on the first non-blank character after the paragraph if there is one, or
 
@@ -374,6 +414,7 @@ static int is_part_of_paragraph(const line_desc * const ld, const int tab_size,
 int paragraph(buffer * const b) {
 	int pos,
 	    done,
+	    skip,
 	    line = b->cur_line,
 	    right_margin = b->opt.right_margin ? b->opt.right_margin : ne_columns;
 
@@ -395,7 +436,7 @@ int paragraph(buffer * const b) {
 	start_undo_chain(b);
 
 	/* This useless insertion and deletion of a single character ensures
-	that the text isn't shifted way over to the left after an undo. */
+	   that the text isn't shifted way over to the left after an undo. */
 	insert_one_char(b, ld, line, 0, ' ');
 	delete_stream(b, ld, line, 0, 1);
 
@@ -405,15 +446,18 @@ int paragraph(buffer * const b) {
 		trim_trailing_space(b, ld, line, b->encoding);
 
 		/** Step 4 **/
-		while (!done && calc_width(ld, ld->line_len, b->opt.tab_size, b->encoding) > right_margin) {
+		while (!stop && !done && calc_width(ld, ld->line_len, b->opt.tab_size, b->encoding) > right_margin) {
 			int spaces;
 			int split_pos;
 			int did_split;
 
 			/** 4.1  Find the split point **/
 
-			pos = 0;   /* Skip past leading spaces. */
+			pos = 0;   /* Skip past leading spaces... */
 			while(pos < ld->line_len && isasciispace(ld->line[pos]))
+			  pos = next_pos(ld->line, pos, b->encoding);
+			           /* ...and the invariants if any. */
+			while(pos < ld->line_len && isparaspot(ld->line[pos]))
 			  pos = next_pos(ld->line, pos, b->encoding);
 
 			did_split = split_pos = spaces = 0;
@@ -445,12 +489,16 @@ int paragraph(buffer * const b) {
 				ld = (line_desc *)ld->ld_node.next;
 				line++;
 
-				/** 4.5  Insert the ps_space[] stream we saved in step 1. Note that  **/
+				/** 4.5  Insert the pa_space[] stream we saved in step 1. Note that  **/
 				/** we only want to do this if this line is the result of a split,   **/
 				/** which is true if did_split != 0.                                 **/
 
-				if (did_split && pa_space && pa_space_len) {
-					insert_stream(b, ld, line, 0, pa_space, pa_space_pos);
+				if (did_split) {
+					if (pa_space && pa_space_len && pa_space_pos)
+						insert_stream(b, ld, line, 0, pa_space, pa_space_pos);
+					/** 4.5.1 Insert the pa_spots[] stream if there is one. **/
+					if (pa_spots && pa_spots_pos)
+						insert_stream(b, ld, line, pa_space_pos, pa_spots, pa_spots_pos);
 				}
 
 				/** 4.6  Trim trailing space off the end of the line. **/
@@ -459,22 +507,45 @@ int paragraph(buffer * const b) {
 			else done = TRUE;
 		}
 
+		/** If the current line is just a spot (no text), we skip over it.      **/
+		pos = 0;   /* Skip past leading spaces... */
+		skip = FALSE;
+		while(pos < ld->line_len && isasciispace(ld->line[pos]))
+			pos = next_pos(ld->line, pos, b->encoding);
+		           /* ...and the invariants if any. */
+		while(pos < ld->line_len && isparaspot(ld->line[pos]))
+			pos = next_pos(ld->line, pos, b->encoding);
+		if (pos == ld->line_len) skip = TRUE;
+			
 		/** 5. If the _following_ line is part of this paragraph (i.e., its first  **/
 		/**     non-blank character is in the correct position):                   **/
 
 		if (ld->ld_node.next->next && is_part_of_paragraph((line_desc *)ld->ld_node.next, b->opt.tab_size, &pos, b->encoding)) {
-			/** 5.1  Add a space to the end of the current line.                    **/
-			insert_one_char(b, ld, line, ld->line_len, ' ');
+			/** If the next line is just a spot (no text), we want to skip over it  **/
+			/** rather than splicing it to the current line.                        **/
+			if (skip || save_spots((line_desc *)ld->ld_node.next, pos, b->encoding) && ((line_desc *)ld->ld_node.next)->line_len <= pos+pa_spots_pos) {
+				/* skip to next line */
+				ld = (line_desc *)ld->ld_node.next;
+				line++;
+			}
+			else {
+				/** 5.1  Add a space to the end of the current line.                    **/
+				insert_one_char(b, ld, line, ld->line_len, ' ');
 
-			/** 5.2  Move following line's data starting with the first             **/
-			/** non-blank to the end of the current line.                           **/
-			/**  We do this by first deleting the leading spaces, then we           **/
-			/**  delete the newline at the end of the current line.                 **/
-			if (pos > 0) delete_stream(b, (line_desc *)ld->ld_node.next, line + 1, 0, pos);
-			delete_stream(b, ld, line, ld->line_len, 1);
+				/** 5.4  Move following line's data starting with the first             **/
+				/** non-blank to the end of the current line.                           **/
+
+				/**  We do this by first deleting the leading spaces                    **/
+				if (pos > 0) delete_stream(b, (line_desc *)ld->ld_node.next, line + 1, 0, pos);
+				/** 5.2 Cache the leading non-alphanumeric in pa_spots, then delete it,  **/
+				if (save_spots((line_desc *)ld->ld_node.next, 0, b->encoding))
+					delete_stream(b, (line_desc *)ld->ld_node.next, line + 1, 0, pa_spots_pos);
+				/**  Finally splice the lines by deleting the newline at the end of the current line.                 **/
+				delete_stream(b, ld, line, ld->line_len, 1);
+			}
 		}
 		else done = TRUE;
-	} while (!done);
+	} while (!done && !stop);
 
 	/** Step 6 **/
 	end_undo_chain(b);
@@ -495,7 +566,7 @@ int paragraph(buffer * const b) {
 
 	/** Step 9 **/
 	goto_line(b, line);
-	if (line_down(b) == ERROR) return ERROR;
+	if (stop || line_down(b) == ERROR) return stop ? STOPPED : ERROR;
 
 	/* Try to find the first non-blank starting with this line. */
 	ld = b->cur_line_desc;
@@ -520,7 +591,7 @@ int paragraph(buffer * const b) {
 
 
 /* Centers the current line with respect to the right_margin parameter. If the
- line (without spaces) is longer than the right margin, nothing happens. */
+   line (without spaces) is longer than the right margin, nothing happens. */
 
 int center(buffer * const b) {
 
@@ -553,8 +624,8 @@ int center(buffer * const b) {
 
 
 /* Indents a line of the amount of whitespace present on the previous line, stopping
-	at a given column (use INT_MAX for not stopping). The number of
-	inserted bytes is returned. */
+   at a given column (use INT_MAX for not stopping). The number of
+   inserted bytes is returned. */
 
 int auto_indent_line(buffer * const b, const int line, line_desc * const ld, const int up_to_col) {
 
@@ -572,7 +643,7 @@ int auto_indent_line(buffer * const b, const int line, line_desc * const ld, con
 		if (col > up_to_col) break;
 		pos = next_pos(prev_ld->line, pos, b->encoding);
 	}
-	insert_stream(b, ld, line, 0, prev_ld->line, pos);
+	if (pos) insert_stream(b, ld, line, 0, prev_ld->line, pos);
 	return pos;
 }
 
@@ -646,6 +717,7 @@ int shift(buffer * const b, char *p, char *msg, int msg_size) {
 		start_undo_chain(b);
 		for (line=first_line; line<=last_line; line++) {
 			int pos, c_pos, c_col_orig, offset;
+			b->attr_len = -1;
 			goto_line(b,line);
 			ld = b->cur_line_desc;
 			if (line == first_line) start_line_desc = ld;
@@ -699,7 +771,9 @@ int shift(buffer * const b, char *p, char *msg, int msg_size) {
 					char c = ' ';
 					if (use_tabs && (b->opt.tab_size - calc_width(ld, pos, b->opt.tab_size, b->encoding) % b->opt.tab_size) <= -offset )
 						c = '\t';
-					insert_one_char(b, ld, b->cur_line, pos, c);
+					if (insert_one_char(b, ld, b->cur_line, pos, c)) {
+						break;
+					}
 					pos++;
 					c_pos++;
 				}
