@@ -1,7 +1,7 @@
 /* Help strings.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -354,6 +354,17 @@ const char * const DELETEPREVWORD_HELP[8] = {
   "word the effect is just to delete to the start of that word).",
 };
 
+const char * const DELTABS_HELP[9] = {
+  "Syntax: DelTabs [0|1]",
+  "Abbreviation: DT",
+  "",
+  "sets the `DelTabs' flag. When this flag is set, a `d' will appear on",
+  "the status bar, and the <BackSpace> and <Del> keys will remove a tab's",
+  "worth of SPACE characters if a TAB character could have occupied the",
+  "same whitespace in the current line as the removed spaces.  This is the",
+  "deletion counterpart to the `Tabs' flag. `Tabs'.",
+};
+
 const char * const DOUNDO_HELP[21] = {
   "Syntax: DoUndo [0|1]",
   "Abbreviation: DU",
@@ -484,7 +495,7 @@ const char * const FINDREGEXP_HELP[12] = {
   "on the input line, the default being the last pattern used.",
 };
 
-const char * const FLAGS_HELP[33] = {
+const char * const FLAGS_HELP[34] = {
   "Syntax: Flags",
   "Abbreviation: FLAG",
   "",
@@ -502,8 +513,9 @@ const char * const FLAGS_HELP[33] = {
   "       v   VerboseMacros VM   record macros using use long command names",
   "       u   DoUndo        DU   record edits for later undoing",
   "       r   ReadOnly      RO   changes are not allowed",
-  "       t   Tabs          TAB  TAB key inserts TABs instead of spaces",
-  "       T   ShiftTabs     ST   Shift may insert TABs",
+  "       t/T Tabs          TAB  TAB key inserts TABs instead of spaces",
+  "       T   ShiftTabs     ST   Shift may insert TABs (only if 't' is also set)",
+  "       d   DelTabs       DT   BS and DEL may remove tabs worth of space",
   "       B   Binary        B    affects file loading/saving",
   "       M   Mark          M    mark set for line-oriented block operations",
   "       V   MarkVert      MV   like mark, but block is rectangle",
@@ -1065,7 +1077,7 @@ const char * const PAGEUP_HELP[6] = {
   "assumed to be one.",
 };
 
-const char * const PARAGRAPH_HELP[24] = {
+const char * const PARAGRAPH_HELP[27] = {
   "Syntax: Paragraph [N]",
   "Abbreviation: PA",
   "",
@@ -1078,17 +1090,20 @@ const char * const PARAGRAPH_HELP[24] = {
   "(regardless of its leading white space) and all subsequent non-blank",
   "lines that have identical (to each other's--not to the first line's)",
   "leading white space. Therefore your paragraphs can have various first",
-  "line indentations and left margins.",
+  "line indentations and left margins. `Paragraph' also takes into account",
+  "characters commonly used at the left edge of block comments (`/', `*',",
+  "`#', spaces, and tabs) or quoted email (`>'), and attempts to preserve",
+  "those on the left edge when possible.",
   "",
   "   After the `Paragraph' command completes, your cursor will be",
   "positioned on the first non-blank character after the last reformatted",
   "paragraph (or, if there is no such character, at the end of the",
   "document).",
   "",
-  "   If you think paragraphing should insert \"smart\" spaces after full",
-  "stops and colons, and do other \"smart\" things such as justification,",
-  "you should consider using a text formatter. TeX is usually the best",
-  "choice.",
+  "   `Paragraph' does not insert \"smart\" spaces after full stops and",
+  "colons, nor does it do other \"smart\" things such as justification. If",
+  "you need such facilities, you should consider using a text formatter.",
+  "TeX for example is usually an excellent choice.",
 };
 
 const char * const PASTE_HELP[7] = {
@@ -1145,11 +1160,11 @@ const char * const POPPREFS_HELP[24] = {
   "",
   "PopPrefs restores the following values from the preferences stack:",
   "",
-  "       AutoIndent   DoUndo      PreserveCR   ShiftTabs   WordWrap",
-  "       AutoPrefs    FreeForm    ReadOnly     Tabs",
-  "       Binary       HexCode     RightMargin  TabSize",
-  "       CaseSearch   Insert      SearchBack   UTF8Auto",
-  "       ClipNumber   NoFileReq   StatusBar    VisualBell",
+  "       AutoIndent  DelTabs   NoFileReq    StatusBar  VisualBell",
+  "       AutoPrefs   DoUndo    PreserveCR   ShiftTabs  WordWrap",
+  "       Binary      FreeForm  ReadOnly     Tabs",
+  "       CaseSearch  HexCode   RightMargin  TabSize",
+  "       ClipNumber  Insert    SearchBack   UTF8Auto",
 };
 
 const char * const PRESERVECR_HELP[15] = {
@@ -1215,11 +1230,11 @@ const char * const PUSHPREFS_HELP[24] = {
   "",
   "`PushPrefs' saves the following values on the preferences stack:",
   "",
-  "       AutoIndent   DoUndo      PreserveCR   ShiftTabs   WordWrap",
-  "       AutoPrefs    FreeForm    ReadOnly     Tabs",
-  "       Binary       HexCode     RightMargin  TabSize",
-  "       CaseSearch   Insert      SearchBack   UTF8Auto",
-  "       ClipNumber   NoFileReq   StatusBar    VisualBell",
+  "       AutoIndent  DelTabs   NoFileReq    StatusBar  VisualBell",
+  "       AutoPrefs   DoUndo    PreserveCR   ShiftTabs  WordWrap",
+  "       Binary      FreeForm  ReadOnly     Tabs",
+  "       CaseSearch  HexCode   RightMargin  TabSize",
+  "       ClipNumber  Insert    SearchBack   UTF8Auto",
 };
 
 const char * const QUIT_HELP[6] = {
@@ -1568,7 +1583,7 @@ const char * const SETBOOKMARK_HELP[12] = {
   "is issued.",
 };
 
-const char * const SHIFT_HELP[22] = {
+const char * const SHIFT_HELP[21] = {
   "Syntax: Shift [<|>][n][t|s]]",
   "Abbreviation: SH",
   "",
@@ -1581,24 +1596,25 @@ const char * const SHIFT_HELP[22] = {
   "affected.",
   "",
   "   `Shift' will insert tab characters only if the document's `Tabs'",
-  "flag is set--in which case a `t' appears in the status bar-- and the",
-  "`ShiftTabs' is also set--in which case the `t' in the status bar will",
-  "be upper case. If either of the `ShiftTabs' or `Tabs' flags are off",
-  "(and therefore the `t' in the status bar is lower case or missing,",
-  "respectively) `Shift' will only insert spaces.",
+  "flag and the `ShiftTabs' flag are both set--in which case an upper case",
+  "`T' will appear in the status bar. If either of the `Tabs' or",
+  "`ShiftTabs' flags is unset (i.e there is no upper case `T' in the",
+  "status bar) `Shift' will only insert spaces.",
   "",
   "   In the case of left shifts, if any indicated line has insufficient",
   "leading white space for the requested adjustment to be made, then",
   "`Shift' reports an error and makes no changes.",
 };
 
-const char * const SHIFTTABS_HELP[7] = {
+const char * const SHIFTTABS_HELP[9] = {
   "Syntax: ShiftTabs [0|1]",
   "Abbreviation: SHT",
   "",
-  "sets the `ShiftTabs' flag. When this flag and the `Tabs' flag are both",
-  "true, left and right `Shift' commands may use tab characters to adjust",
-  "leading white space. Otherwise only spaces are used.  `Shift'.",
+  "sets the `ShiftTabs' flag. `ShiftTabs' has an effect only when the",
+  "`Tabs' flag is set, in which case an upper case `T' appears in the",
+  "status line. When this flag and the `Tabs' flag are both set, left and",
+  "right `Shift' commands may use tab characters to adjust leading white",
+  "space. Otherwise only spaces are used. `Shift'.",
 };
 
 const char * const STATUSBAR_HELP[23] = {
@@ -1691,30 +1707,23 @@ const char * const SYSTEM_HELP[10] = {
   "on the input line.",
 };
 
-const char * const TABS_HELP[24] = {
+const char * const TABS_HELP[17] = {
   "Syntax: Tabs [0|1]",
   "Abbreviation: TAB",
   "",
-  "sets the `Tabs' flag. When this flag is true, the `InsertTab' command",
-  "will insert literal TAB characters. Otherwise it will insert enough",
-  "spaces to have the same visual effect.",
+  "sets the `Tabs' flag. When this flag is set, <Tab> key and the",
+  "`InsertTab' command will insert literal TAB characters.  Otherwise it",
+  "will insert enough spaces to have the same visual effect.",
   "",
   "   In normal editing, the <Tab> key invokes the command \"`InsertTab'",
   "1\". Unlike most others, the <Tab> key cannot be mapped to other",
   "commands. Thus the `Tabs' flag provides the only customization `ne'",
   "offers for the <Tab> key.",
   "",
-  "   The `Tabs' flag also affects the action of the <BackSpace> and",
-  "<Delete> keys in one special circumstance. That is, if the normal",
-  "action of these keys would remove white space, and that white space",
-  "could be represented by a tab, then one tab's worth of space is",
-  "removed. In all other cases, the <BackSpace> and <Delete> keys perform",
-  "their normal function.",
-  "",
-  "   If you invoke `Tabs' with no arguments, it will toggle the flag. If",
-  "you specify 0 or 1, the flag will be set to false or true,",
-  "respectively. A lower case `t' will appear on the status bar if the",
-  "flag is true.",
+  "   If set, either a lower case `t' or upper case `T' will appear in the",
+  "status bar depending on the state of the `ShiftTabs' flag.  (The",
+  "`ShiftTabs' flag is irrelevant if the `Tabs' flag is off.)  ",
+  "`ShiftTabs'.",
 };
 
 const char * const TABSIZE_HELP[9] = {
@@ -1967,13 +1976,16 @@ const char * const VISUALBELL_HELP[10] = {
   "respectively.",
 };
 
-const char * const WORDWRAP_HELP[12] = {
+const char * const WORDWRAP_HELP[15] = {
   "Syntax: WordWrap [0|1]",
   "Abbreviation: WW",
   "",
-  "sets the word wrap flag. When this flag is true, `ne' will automatically",
-  "break lines of text longer than the current right margin while you type",
-  "them. See `RightMargin'.",
+  "sets the word wrap flag. When this flag is true, `ne' will",
+  "automatically break lines of text when you attempt to insert characters",
+  "beyond the right margin. See `RightMargin'. `ne' will attempt to",
+  "preserve certain invariant characters at the left edge of paragraphs",
+  "typically used to indicate comments in source code or quoted passages",
+  "in email text. See `Paragraph'.",
   "",
   "   If you invoke `WordWrap' with no arguments, it will toggle the flag.",
   "If you specify 0 or 1, the flag will be set to false or true,",
