@@ -1,7 +1,7 @@
 /* Extern's for names and abbreviations of all the commands.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -94,6 +94,9 @@ extern const char DELETENEXTWORD_ABBREV[];
 extern const char DELETEPREVWORD_NAME[];
 extern const char DELETEPREVWORD_ABBREV[];
 
+extern const char DELTABS_NAME[];
+extern const char DELTABS_ABBREV[];
+
 extern const char DOUNDO_NAME[];
 extern const char DOUNDO_ABBREV[];
 
