@@ -1,7 +1,7 @@
 /* main(), global initialization and global buffer functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna
-	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -36,7 +36,7 @@ message disappears as soon as any key is typed. */
 
 char *NO_WARRANTY_msg[] = {	PROGRAM_NAME " " VERSION ".",
 										"Copyright (C) 1993-1998 Sebastiano Vigna",
-										"Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna",
+										"Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna",
 										"",
 										"This program is free software; you can redistribute it and/or modify",
 										"it under the terms of the GNU General Public License as published by",
@@ -56,7 +56,7 @@ char *NO_WARRANTY_msg[] = {	PROGRAM_NAME " " VERSION ".",
 										"by [ are activated by Control+Meta or just Meta, depending on your terminal",
 										"emulator. Alternatively, just press Escape followed by a letter.",
 										"",
-										"ne Home page: http://ne.dsi.unimi.it/",
+										"ne home page: http://ne.di.unimi.it/",
 										"Discuss ne at http://groups.google.com/group/niceeditor/",
 										NULL
 									};
