@@ -1,7 +1,7 @@
 /* Main typedefs and defines.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -128,9 +128,14 @@ typedef enum {
 	ENC_ASCII = 0, ENC_UTF8, ENC_8_BIT
 } encoding_type;
 
-/* The number of bookmarks per buffer. It comprises the automatic bookmark (0). */
+/* Bookmark designations. User get 0 through 9 plus the AUTO_BOOKMARK '-'. */
 
-#define NUM_BOOKMARKS      11
+enum {
+	MAX_USER_BOOKMARK = 9,
+	AUTO_BOOKMARK,
+	WORDWRAP_BOOKMARK,
+	NUM_BOOKMARKS
+};
 
 /* These are the list and node structures used throughout ne. See the exec.c
    source for some elaborations on the subject. */
@@ -380,7 +385,8 @@ typedef struct {
 		read_only:1,       /* Read-only mode */
 		search_back:1,     /* Last search was backwards */
 		case_search:1,     /* Look at case matching in searches */
-		tabs:1,            /* TAB inserts TABs vs. spaces */
+		tabs:1,            /* TAB inserts TABs(1) vs. spaces(0) */
+		del_tabs:1,        /* DEL/BS deletes tab's worth of space. */
 		shift_tabs:1,      /* Shift may insert tabs, but only if tabs is also true */
 		automatch:4,       /* Automatically match visible brackets */
 		binary:1,          /* Load and save in binary mode */
