@@ -1,7 +1,7 @@
 /* Miscellaneous support functions.
 
 	Copyright (C) 1993-1998 Sebastiano Vigna 
-	Copyright (C) 1999-2012 Todd M. Lewis and Sebastiano Vigna
+	Copyright (C) 1999-2013 Todd M. Lewis and Sebastiano Vigna
 
 	This file is part of ne, the nice editor.
 
@@ -375,7 +375,7 @@ int calc_width(const line_desc * const ld, const int n, const int tab_size, cons
 
 	int pos, len;
 
-	for(pos = len = 0; pos < n; pos = next_pos(ld->line, pos, encoding)) {
+	for(pos = len = 0; pos < n; pos = pos < ld->line_len ? next_pos(ld->line, pos, encoding) : pos + 1) {
 		if (pos >= ld->line_len) len++;
 		else if (ld->line[pos] != '\t') len += get_char_width(&ld->line[pos], encoding);
 		else len += tab_size - len % tab_size;
@@ -410,6 +410,17 @@ int calc_pos(const line_desc * const ld, const int col, const int tab_size, cons
 	return pos;
 }
 
+/* Returns true if the specified character is invariant on the left edge of re-wrapped paragraphs */
+
+int isparaspot(const int c) {
+	char *spots = "/*#>\t ";
+	char *p = spots;
+	while (*p) {
+		if (*p++ == c) return TRUE;
+	}
+	return FALSE;
+}
+
 /* Returns true if the specified character is an US-ASCII whitespace character. */
 
 int isasciispace(const int c) {
